@@ -26,10 +26,14 @@
  * Initialize the plugin.
  *
  * @param cfg configuration to use
+ * @param skip_preflight true if we should skip the usual
+ *   preflight check which assures us that the DB is actually
+ *   operational; only sync-dbinit should use true here.
  * @return NULL on failure
  */
 struct SYNC_DatabasePlugin *
-SYNC_DB_plugin_load (const struct GNUNET_CONFIGURATION_Handle *cfg);
+SYNC_DB_plugin_load (const struct GNUNET_CONFIGURATION_Handle *cfg,
+                     bool skip_preflight);
 
 
 /**
