@@ -173,19 +173,6 @@ struct SYNC_UploadDetails
 
     } success;
 
-    /**
-     * Data returned if @e us is #SYNC_US_NOT_MODIFIED.
-     */
-    struct
-    {
-
-      /**
-       * Hash of the synchronized backup.
-       */
-      const struct GNUNET_HashCode *curr_backup_hash;
-
-    } not_modified;
-
     /**
      * Previous backup. Returned if @e us is
      * #SYNC_US_CONFLICTING_BACKUP
@@ -290,8 +277,8 @@ enum SYNC_PaymentOptions
  * @param backup_size number of bytes in @a backup
  * @param backup the encrypted backup, must remain in
  *         memory until we are done with the operation!
- * @param payment_requested #GNUNET_YES if the client wants to pay more for the account now
  * @param po payment options
+ * @param paid_order_id ID of the paid order, NULL if no payment was made so far
  * @param cb function to call with the result
  * @param cb_cls closure for @a cb
  * @return handle for the operation
@@ -314,7 +301,7 @@ SYNC_upload (struct GNUNET_CURL_Context *ctx,
  * that it did not complete, it is possible that the server did
  * receive the full request before the upload is aborted.
  *
- * @param uo operation to cancel.
+ * @param[in] uo operation to cancel.
  */
 void
 SYNC_upload_cancel (struct SYNC_UploadOperation *uo);
@@ -412,7 +399,7 @@ SYNC_download (struct GNUNET_CURL_Context *ctx,
 /**
  * Cancel the download.
  *
- * @param do operation to cancel.
+ * @param[in] download operation to cancel.
  */
 void
 SYNC_download_cancel (struct SYNC_DownloadOperation *download);
