@@ -27,7 +27,7 @@
  * Return default project data used by Sync.
  */
 const struct GNUNET_OS_ProjectData *
-SYNC_project_data_default (void);
+SYNC_project_data (void);
 
 
 #endif
