@@ -28,11 +28,6 @@
 #include "sync-httpd_backup.h"
 #include "sync-httpd_config.h"
 
-/**
- * Backlog for listen operation on unix-domain sockets.
- */
-#define UNIX_BACKLOG 500
-
 
 /**
  * Should a "Connection: close" header be added to each HTTP response?
@@ -69,20 +64,15 @@ struct GNUNET_CURL_Context *SH_ctx;
  */
 static struct GNUNET_CURL_RescheduleContext *rc;
 
-/**
- * Task running the HTTP server.
- */
-static struct GNUNET_SCHEDULER_Task *mhd_task;
-
 /**
  * Global return code
  */
 static int global_ret;
 
 /**
- * The MHD Daemon
+ * Set to true if we have started an MHD daemons.
  */
-static struct MHD_Daemon *mhd;
+static bool have_daemons;
 
 /**
  * Connection handle to the our database
@@ -335,11 +325,7 @@ do_shutdown (void *cls)
 {
   (void) cls;
   SH_resume_all_bc ();
-  if (NULL != mhd_task)
-  {
-    GNUNET_SCHEDULER_cancel (mhd_task);
-    mhd_task = NULL;
-  }
+  TALER_MHD_daemons_halt ();
   if (NULL != SH_ctx)
   {
     GNUNET_CURL_fini (SH_ctx);
@@ -350,11 +336,7 @@ do_shutdown (void *cls)
     GNUNET_CURL_gnunet_rc_destroy (rc);
     rc = NULL;
   }
-  if (NULL != mhd)
-  {
-    MHD_stop_daemon (mhd);
-    mhd = NULL;
-  }
+  TALER_MHD_daemons_destroy ();
   if (NULL != db)
   {
     SYNC_DB_plugin_unload (db);
@@ -397,39 +379,6 @@ handle_mhd_completion_callback (void *cls,
 }
 
 
-/**
- * Function that queries MHD's select sets and
- * starts the task waiting for them.
- */
-static struct GNUNET_SCHEDULER_Task *
-prepare_daemon (void);
-
-
-/**
- * Set if we should immediately #MHD_run again.
- */
-static int triggered;
-
-
-/**
- * Call MHD to process pending requests and then go back
- * and schedule the next run.
- *
- * @param cls the `struct MHD_Daemon` of the HTTP server to run
- */
-static void
-run_daemon (void *cls)
-{
-  (void) cls;
-  mhd_task = NULL;
-  do {
-    triggered = 0;
-    GNUNET_assert (MHD_YES == MHD_run (mhd));
-  } while (0 != triggered);
-  mhd_task = prepare_daemon ();
-}
-
-
 /**
  * Kick MHD to run now, to be called after MHD_resume_connection().
  * Basically, we need to explicitly resume MHD's event loop whenever
@@ -439,16 +388,7 @@ run_daemon (void *cls)
 void
 SH_trigger_daemon ()
 {
-  if (NULL != mhd_task)
-  {
-    GNUNET_SCHEDULER_cancel (mhd_task);
-    mhd_task = GNUNET_SCHEDULER_add_now (&run_daemon,
-                                         NULL);
-  }
-  else
-  {
-    triggered = 1;
-  }
+  TALER_MHD_daemon_trigger ();
 }
 
 
@@ -463,55 +403,40 @@ SH_trigger_curl ()
 
 
 /**
- * Function that queries MHD's select sets and
- * starts the task waiting for them.
+ * Callback invoked on every listen socket to start the
+ * respective MHD HTTP daemon.
  *
- * @return task that runs the HTTP server
+ * @param cls unused
+ * @param lsock the listen socket
  */
-static struct GNUNET_SCHEDULER_Task *
-prepare_daemon (void)
+static void
+start_daemon (void *cls,
+              int lsock)
 {
-  struct GNUNET_SCHEDULER_Task *ret;
-  fd_set rs;
-  fd_set ws;
-  fd_set es;
-  struct GNUNET_NETWORK_FDSet *wrs;
-  struct GNUNET_NETWORK_FDSet *wws;
-  int max;
-  MHD_UNSIGNED_LONG_LONG timeout;
-  int haveto;
-  struct GNUNET_TIME_Relative tv;
+  struct MHD_Daemon *mhd;
 
-  FD_ZERO (&rs);
-  FD_ZERO (&ws);
-  FD_ZERO (&es);
-  wrs = GNUNET_NETWORK_fdset_create ();
-  wws = GNUNET_NETWORK_fdset_create ();
-  max = -1;
-  GNUNET_assert (MHD_YES ==
-                 MHD_get_fdset (mhd,
-                                &rs,
-                                &ws,
-                                &es,
-                                &max));
-  haveto = MHD_get_timeout (mhd, &timeout);
-  if (haveto == MHD_YES)
-    tv.rel_value_us = (uint64_t) timeout * 1000LL;
-  else
-    tv = GNUNET_TIME_UNIT_FOREVER_REL;
-  GNUNET_NETWORK_fdset_copy_native (wrs, &rs, max + 1);
-  GNUNET_NETWORK_fdset_copy_native (wws, &ws, max + 1);
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Adding run_daemon select task\n");
-  ret = GNUNET_SCHEDULER_add_select (GNUNET_SCHEDULER_PRIORITY_HIGH,
-                                     tv,
-                                     wrs,
-                                     wws,
-                                     &run_daemon,
-                                     NULL);
-  GNUNET_NETWORK_fdset_destroy (wrs);
-  GNUNET_NETWORK_fdset_destroy (wws);
-  return ret;
+  (void) cls;
+  GNUNET_assert (-1 != lsock);
+  mhd = MHD_start_daemon (MHD_USE_SUSPEND_RESUME | MHD_USE_DUAL_STACK,
+                          0 /* port */,
+                          NULL, NULL,
+                          &url_handler, NULL,
+                          MHD_OPTION_LISTEN_SOCKET, lsock,
+                          MHD_OPTION_NOTIFY_COMPLETED,
+                          &handle_mhd_completion_callback, NULL,
+                          MHD_OPTION_CONNECTION_TIMEOUT,
+                          (unsigned int) 10 /* 10s */,
+                          MHD_OPTION_END);
+  if (NULL == mhd)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to launch HTTP service, exiting.\n");
+    global_ret = EXIT_NO_RESTART;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
+  have_daemons = true;
+  TALER_MHD_daemon_start (mhd);
 }
 
 
@@ -530,9 +455,7 @@ run (void *cls,
      const char *cfgfile,
      const struct GNUNET_CONFIGURATION_Handle *config)
 {
-  int fh;
   enum TALER_MHD_GlobalOptions go;
-  uint16_t port;
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Starting sync-httpd\n");
@@ -656,36 +579,34 @@ run (void *cls,
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
-  fh = TALER_MHD_bind (config,
-                       "sync",
-                       &port);
-  if ( (0 == port) &&
-       (-1 == fh) )
   {
-    global_ret = EXIT_NO_RESTART;
-    GNUNET_SCHEDULER_shutdown ();
-    return;
-  }
-  mhd = MHD_start_daemon (MHD_USE_SUSPEND_RESUME | MHD_USE_DUAL_STACK,
-                          port,
-                          NULL, NULL,
-                          &url_handler, NULL,
-                          MHD_OPTION_LISTEN_SOCKET, fh,
-                          MHD_OPTION_NOTIFY_COMPLETED,
-                          &handle_mhd_completion_callback, NULL,
-                          MHD_OPTION_CONNECTION_TIMEOUT,
-                          (unsigned int) 10 /* 10s */,
-                          MHD_OPTION_END);
-  if (NULL == mhd)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Failed to launch HTTP service, exiting.\n");
-    global_ret = EXIT_NO_RESTART;
-    GNUNET_SCHEDULER_shutdown ();
-    return;
+    enum GNUNET_GenericReturnValue ret;
+
+    ret = TALER_MHD_listen_bind (config,
+                                 "sync",
+                                 &start_daemon,
+                                 NULL);
+    switch (ret)
+    {
+    case GNUNET_SYSERR:
+      global_ret = EXIT_NOTCONFIGURED;
+      GNUNET_SCHEDULER_shutdown ();
+      return;
+    case GNUNET_NO:
+      if (! have_daemons)
+      {
+        global_ret = EXIT_NOTCONFIGURED;
+        GNUNET_SCHEDULER_shutdown ();
+        return;
+      }
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "Could not open all configured listen sockets\n");
+      break;
+    case GNUNET_OK:
+      break;
+    }
   }
   global_ret = EXIT_SUCCESS;
-  mhd_task = prepare_daemon ();
 }
 
 
