@@ -207,7 +207,8 @@ make_payment_request (const char *order_id,
               "Creating payment request for order `%s'\n",
               order_id);
   resp = MHD_response_from_empty (MHD_HTTP_STATUS_PAYMENT_REQUIRED);
-  TALER_MHD2_add_global_headers (resp);
+  TALER_MHD2_add_global_headers (resp,
+                                 false);
   {
     char *hdr;
     const char *pfx;
@@ -607,7 +608,8 @@ handle_database_error (struct BackupContext *bc,
       order_id = MHD_request_get_value (bc->request,
                                         MHD_VK_GET_ARGUMENT,
                                         "paying");
-      if (NULL == order_id)
+      if ( (NULL == order_id) ||
+           (NULL == order_id->cstr) )
       {
         GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                     "Payment required, starting payment process\n");
@@ -749,7 +751,8 @@ handle_upload (void *upload_cls,
       struct MHD_Response *resp;
 
       resp = MHD_response_from_empty (MHD_HTTP_STATUS_NOT_MODIFIED);
-      TALER_MHD2_add_global_headers (resp);
+      TALER_MHD2_add_global_headers (resp,
+                                     false);
       return MHD_upload_action_from_response (request,
                                               resp);
     }
@@ -760,7 +763,8 @@ handle_upload (void *upload_cls,
     struct MHD_Response *resp;
 
     resp = MHD_response_from_empty (MHD_HTTP_STATUS_NO_CONTENT);
-    TALER_MHD2_add_global_headers (resp);
+    TALER_MHD2_add_global_headers (resp,
+                                   false);
     return MHD_upload_action_from_response (request,
                                             resp);
   }
@@ -944,7 +948,8 @@ SH_backup_post (struct MHD_Request *request,
       struct MHD_Response *resp;
 
       resp = MHD_response_from_empty (MHD_HTTP_STATUS_NOT_MODIFIED);
-      TALER_MHD2_add_global_headers (resp);
+      TALER_MHD2_add_global_headers (resp,
+                                     false);
       return MHD_action_from_response (request,
                                        resp);
     }
